package com.sillycycle.bagleyd.triangles;

/*
 * @(#)TrianglesFrame.java
 *
 * Copyright 1994 - 2022  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.util.ArgumentParser;
import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.ComponentUtil;
import com.sillycycle.bagleyd.util.Icon;
import com.sillycycle.bagleyd.util.IODialog;
import com.sillycycle.bagleyd.util.OKDialog;
import com.sillycycle.bagleyd.util.Spinner;

/**
 * The <code>TrianglesFrame</code> class is to be run as an application.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

public class TrianglesFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 42L;
	public static final String NL = System.getProperty("line.separator");
	static final int MAX_TILES = 10;
	static final int MIN_SPEED = 1;
	static final int MAX_SPEED = 50;
	static final int NEVER = -1;
	static final String SCOREFILE = "triangles.scores";
	static final String[] COMMAND_STRINGS = new String[] {
		"Get Buffer", "Write Buffer", "Show Buffer",
		"Undo", "Redo", "Clear",
		"Randomize", "(Auto-solve)",
		"Corners Mode",
		"Speed >", "Slow <", "Sound @",
		"Description ?", "Features !", "References ^", "About",
		"Get File", "Write File", "Exit"};
	static final char[] COMMAND_CHARS = new char[] {
		'g', 'w', '*',
		'u', 'r', 'c', 'z', 's',
		'm', '>', '<', '@',
		'?', '!', '^', 'a', 'G', 'W', 'x'};

	static ArgumentParser argumentParser = null;
	Icon mediumIcon = null;
	Icon largeIcon = null;
	int widthValue = 320;
	int heightValue = 275;
	static int xValue = 0;
	static int yValue = 0;
	Color tileColor = Color.lightGray;
	Color bgColor = Color.gray;
	boolean sound = false;
	boolean controlValue = true;
	boolean cornersValue = TrianglesInterface.DEFAULT_CORNERS;
	int tileXValue = TrianglesInterface.DEFAULT_TILESX;
	int tileYValue = TrianglesInterface.DEFAULT_TILESY;
	int baseValue = TrianglesInterface.DEFAULT_BASE;
	int delayValue = 0; //10;
	int speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
	boolean debug = false;
	boolean randomized = false;
	JPanel topPanel = new JPanel();
	JCheckBox checkBoxCorners = new JCheckBox("Corners");
	Triangles puzzle;
	int movesDisplay = 0;
	String recordDisplay = "NEVER";
	String messageDisplay = "Play Triangles! (use mouse or keypad)";
	JTextField movesValueLabel = new JTextField(Integer.toString(
		movesDisplay));
	JTextField recordValueLabel = new JTextField(recordDisplay);
	Label messageLabel = new Label(messageDisplay, Label.CENTER);
	// JLabel constantly resizes
	OKDialog aDialog = null;
	StringBuilder stringBuilderSave;
	IODialog ioDialog = null;
	int[][][] puzzleRecord = null;
	JMenuBar menuBar;
	JPanel mainPanel = new JPanel();
	JPanel secondaryPanel = new JPanel();
	JFrame frame;
	JCheckBoxMenuItem soundMenuItem = null;
	Spinner tileXChanger = null, tileYChanger = null, speedChanger = null;
	//Slider tileXChanger = null, tileYChanger = null, speedChanger = null;

	public String writePuzzleString() {
		return this.stringBuilderSave.toString();
	}

	public void getPuzzleString(String s) {
		puzzle.stringSave = s;
	}

	public void processKey(char key, boolean control) {
		puzzle.processKey(key, control);
	}

	public boolean getToggleSound() {
		return sound;
	}

	public void toggleSound() {
		sound = !sound;
		if (controlValue)
			soundMenuItem.setSelected(sound);
	}

	public String getOpt(String string) {
		return argumentParser.getOption(string);
	}

	public int getOpt(String param, int value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public boolean getOpt(String param, boolean value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public Color getOpt(String param, Color value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public void getOpts() {
		widthValue = getOpt("windowWidth", widthValue);
		heightValue = getOpt("windowHeight", heightValue);
		xValue = getOpt("windowX", xValue);
		yValue = getOpt("windowY", yValue);
		tileColor = getOpt("tileColor", tileColor);
		bgColor = getOpt("bgColor", bgColor);
		sound = getOpt("sound", sound);
		controlValue = getOpt("control", controlValue);
		tileXValue = getOpt("sizex", tileXValue);
		if (tileXValue < TrianglesInterface.MIN_TILES)
			tileXValue = TrianglesInterface.DEFAULT_TILESX;
		tileYValue = getOpt("sizey", tileYValue);
		if (tileYValue < TrianglesInterface.MIN_TILES)
			tileYValue = TrianglesInterface.DEFAULT_TILESY;
		cornersValue = getOpt("corners", cornersValue);
		baseValue = getOpt("base", baseValue);
		if (baseValue < TrianglesInterface.MIN_BASE ||
				baseValue > TrianglesInterface.MAX_BASE)
			baseValue = TrianglesInterface.DEFAULT_BASE;
		delayValue = getOpt("delay", delayValue);
	}

	public void showBuffer() {
		if (ioDialog != null) {
			ioDialog.setVisible(true);
		}
	}

	public void showMessage(String msg) {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Triangles", msg, "OK",
				mediumIcon, false);
			aDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2, 350, 120);
			aDialog.setVisible(true);
		}
	}

	public void descriptionHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Description",
"Triangles - A sliding tile puzzle.  Triangular version of the 15" + NL +
"puzzle which was originally made around 1879 (the 14-15 puzzle of" + NL +
"Sam Loyd was made soon after).  T. R. Dawson in Original Puzzles" + NL +
"has a similar puzzle with 4 diamond- and 6 triangular-shaped pieces." + NL +
"This puzzle is easier to solve than the 15 puzzle with corner mode" + NL +
"on and proably about the same with corner mode off." + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void featuresHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Features",
"If corners mode on: click \"mouse-left\" button to move a tile." + NL +
"Notice that one can move more than one tile at a time by" + NL +
"clicking on the tile farthest from the space but in line with the" + NL +
"spaces.  Clicks on the spaces or clicks on tiles that are not in" + NL +
"line with the spaces, will not move tiles." + NL +
"If corners mode off: click \"mouse-left\" button to move a tile." + NL +
"One can only move one tile at a time.  The only tiles that can be" + NL +
"Clicks on spaces or clicks tiles that are not in line with a" + NL +
"space, will not move tiles." + NL +
" " + NL +
"Double click on \"mouse-right\" button or press \"C\" or \"c\" keys," + NL +
"clear the puzzle." + NL +
" " + NL +
"Press \"g\" key to get (read) a saved puzzle to screen." + NL +
" " + NL +
"Press \"w\" key to save (write) a puzzle to screen." + NL +
" " + NL +
"Press \"G\" key to get (read) a saved puzzle to file." + NL +
" " + NL +
"Press \"W\" key to save (write) a puzzle to file." + NL +
" " + NL +
"Press \"U\" or \"u\" keys to undo a move." + NL +
" " + NL +
"Press \"R\" or \"r\" keys to redo a move." + NL +
" " + NL +
"Press \"Z\" or \"z\" keys to randomize.  New records can only be set" + NL +
"from a cleared state." + NL +
" " + NL +
"\"S\" or \"s\" keys reserved for the auto-solver (not imple-" + NL +
"mented)." + NL +
" " + NL +
"Press \"M\" or \"m\" keys to toggle corners mode.  (Default is on)." + NL +
" " + NL +
"Press \">\" or \".\" keys to speed up the movement of the tiles." + NL +
" " + NL +
"Press \"<\" or \",\" keys to slow up the movement of the tiles." + NL +
" " + NL +
"Press \"@\" key to toggle the sound." + NL +
" " + NL +
"Press \"Esc\" key to hide program." + NL +
" " + NL +
"Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program." + NL +
" " + NL +
"Use the key pad or arrow keys to move without the" + NL +
"mouse." + NL +
"Key pad (better with Num Lock on) is defined for Triangles as:" + NL +
"7  8  9  Upper Left, Up (no corners), Upper Right" + NL +
"    ^" + NL +
"4< >6  Left, Right" + NL +
"    v" + NL +
"1  2  3  Lower Left, Down (no corners), Lower Right" + NL +
"The arrow keys seem at first to work in the opposite direction" + NL +
"then you expect because the space is not the object moving." + NL +
" " + NL +
"The control key allows you to change the complexity of the" + NL +
"puzzle. Hold down control key and use the keypad or arrow" + NL +
"keys to reduce or enlarge puzzle complexity." + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void referencesHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"References",
"L. E. Horden, Sliding Piece Puzzles (Recreations in " +
"Mathematics Series), Oxford University Press 1986," + NL +
"pp 1, 148, 149." + NL +
"Jerry Slocum & Jack Botermans, Puzzles Old & New (How to Make " +
"and Solve Them), University of Washington" + NL +
"Press, Seattle 1987, p 126, 127." + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void aboutHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"About",
"Java TrianglesFrame Version 8.4.0" + NL +
"Send bugs (reports or fixes) to the author:" +
"David Bagley <bagleyd AT verizon.net>" + NL +
"The latest version is at: " +
"https://www.sillycycle.com/puzzles.html" + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public Icon getIcon(String string, int sx, int sy) {
		URL url = TrianglesFrame.class.getResource("/" + string);

		if (url == null)
			return null;
		return new Icon(Toolkit.getDefaultToolkit().
			getImage(url), sx, sy);
	}

	// TODO Make more reliable, also should cache
	public void playClip(String string) {
		URL url = TrianglesFrame.class.getResource("/" + string);

		if (url == null)
			return;
		try {
			AudioInputStream audioInputStream =
				AudioSystem.getAudioInputStream(url);
			DataLine.Info info = new DataLine.Info(Clip.class,
				audioInputStream.getFormat());
			Clip clip = (Clip)AudioSystem.getLine(info);
			clip.open(audioInputStream);
			clip.start();
		} catch (IOException | LineUnavailableException | UnsupportedAudioFileException e) {
			e.printStackTrace();
		}
	}

	public void playBumpAudio() {
		playClip(TrianglesInterface.SOUNDS[TrianglesInterface.Sounds.BUMP.ordinal()]);
	}

	@Override
	public void setSize(int width, int height) {
		if (puzzle == null)
			return;
		puzzle.setBounds(0, 0, width, height);
		/*repaint();*/ // causes flashing
	}

	@Override
	public void paint(Graphics g) {
		puzzle.setSize(secondaryPanel.getSize().width,
			secondaryPanel.getSize().height - topPanel.getSize().height);
		super.paint(g);
	}

	public String readUserScore() {
		StringBuilder scoreString = new StringBuilder();

		for (int i = 0; i < puzzleRecord.length; i++) {
		  for (int j = 0; j < puzzleRecord[i].length; j++) {
		    for (int k = 0; k < puzzleRecord[i][j].length; k++) {
			scoreString.append(puzzleRecord[i][j][k]);
			if (k < puzzleRecord[i][j].length - 1)
				scoreString.append(" ");
		    }
		    if (j < puzzleRecord[i].length - 1
			|| i < puzzleRecord.length - 1)
		      scoreString.append(NL);
		  }
		  if (i < puzzleRecord.length - 1)
		    scoreString.append(NL);
		}
		return scoreString.toString();
	}

	public void writeScore() {
		try (PrintWriter pr = new PrintWriter(new FileWriter(
				SCOREFILE, false))) {
			pr.println(readUserScore());
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}

	public void initRecords(String cookieValue, boolean store) {
		String[] st = null;
		int field = 0;

		if (cookieValue != null)
			st = cookieValue.split("[+/\\s]+");
		if (debug)
			System.out.println("initRecords: " + cookieValue);
		puzzleRecord = new int[TrianglesInterface.MAX_ORIENT]
			[MAX_TILES - TrianglesInterface.MIN_TILES + 1]
			[MAX_TILES - TrianglesInterface.MIN_TILES + 1];
		for (int i = 0; i < puzzleRecord.length; i++) {
		  for (int j = 0; j < puzzleRecord[i].length; j++) {
		    for (int k = 0; k < puzzleRecord[i][j].length; k++) {
			if (st == null || st.length <= field) {
				//build it from the array
				puzzleRecord[i][j][k] =
					TrianglesScores.PUZZLE_SCORES[i][j][k];
			} else {
				puzzleRecord[i][j][k] =
					Integer.parseInt(st[field++]);
			}
		    }
		  }
		}
		if (store && st == null)
			writeScore(); // if no cookie, then make one...
	}

	public void readScoreRecords() {
		try (BufferedReader br = new BufferedReader(
				new FileReader(SCOREFILE))) {
			StringBuilder scoreData = new StringBuilder();
			String lineData;

			while ((lineData = br.readLine()) != null) {
				scoreData.append(lineData);
				scoreData.append(NL);
			}
			initRecords(scoreData.toString(), true);
		} catch (Exception e) {
			//e.printStackTrace();
			showMessage("Cannot open " + SCOREFILE);
			initRecords(null, true);
		}
	}

	public String readTextArea() {
		return ioDialog.getText();
	}

	public void writeTextArea(String s) {
		ioDialog.setText(s);
	}

	public void shuffleDown() {
		ComponentUtil.findJFrame(this).toBack();
	}

	public void quit() {
		System.exit(0);
	}

	void printRecord(boolean corners,
			int puzzleXDim, int puzzleYDim) {
		int i = (corners) ? TrianglesInterface.CORNERS :
			TrianglesInterface.NOCORN;
		int j = puzzleXDim - TrianglesInterface.MIN_TILES;
		int k = puzzleYDim - TrianglesInterface.MIN_TILES;

		if (puzzleRecord == null)
			return;
		if (puzzleXDim > MAX_TILES || puzzleYDim > MAX_TILES) {
			recordValueLabel.setText("NONE");
		} else if (puzzleRecord[i][j][k] <= NEVER) {
			recordValueLabel.setText("NEVER");
		} else {
			recordValueLabel.setText(Integer.toString(
				puzzleRecord[i][j][k]));
		}
	}

	boolean handleSolved(int counter, boolean corners,
			int puzzleXDim, int puzzleYDim) {
		int i = (corners) ? TrianglesInterface.CORNERS :
			TrianglesInterface.NOCORN;
		int j = puzzleXDim - TrianglesInterface.MIN_TILES;
		int k = puzzleYDim - TrianglesInterface.MIN_TILES;

		if (puzzleRecord != null && tileXValue <= MAX_TILES &&
				tileYValue <= MAX_TILES &&
				(counter < puzzleRecord[i][j][k] ||
				puzzleRecord[i][j][k] <= NEVER)) {
			/* Maybe its been updated by another */
			readScoreRecords();
			puzzleRecord[i][j][k] = counter;
			writeScore();
			printRecord(cornersValue, puzzleXDim, puzzleYDim);
			return true;
		}
		return false;
	}

	public void initScore() {
		if (puzzleRecord == null) {
			readScoreRecords();
			printRecord(cornersValue, tileXValue, tileYValue);
		}
	}

	class TileXChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = tileXValue;

			tileXValue = tileXChanger.getValue();
			if (oldValue != tileXValue) {
				puzzle.setPuzzleSizeX(tileXValue);
				movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));
				printRecord(cornersValue,
					tileXValue, tileYValue);
			}
		}
	}

	class TileYChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = tileYValue;

			tileYValue = tileYChanger.getValue();
			if (oldValue != tileYValue) {
				puzzle.setPuzzleSizeY(tileYValue);
				movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));
				printRecord(cornersValue,
					tileXValue, tileYValue);
			}
		}
	}

	class SpeedChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = speedValue;

			speedValue = speedChanger.getValue();
			if (oldValue != speedValue) {
				puzzle.setDelay(MAX_SPEED + MIN_SPEED - 1 -
					speedValue);
			}
		}
	}

	class CornersItemListener implements ItemListener {
		@Override
		public void itemStateChanged(ItemEvent event) {
			boolean oldValue = cornersValue;

			cornersValue = checkBoxCorners.isSelected();
			if (oldValue != cornersValue) {
				puzzle.setPuzzleMode(cornersValue);
				movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));
				printRecord(cornersValue,
					tileXValue, tileYValue);
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();

		for (int opt = 0; opt < COMMAND_STRINGS.length; opt++) {
			if (command.equals(COMMAND_STRINGS[opt])) {
				processKey(COMMAND_CHARS[opt],
					((event.getModifiers() &
					InputEvent.CTRL_DOWN_MASK) != 0));
				return;
			}
		}
	}

	public void callback(int reason) {
		messageDisplay = "                                                                                ";
		switch(reason) {
		case TrianglesInterface.ACTION_RESTORE:
		case TrianglesInterface.ACTION_RESET:
			movesDisplay = 0;
			randomized = false;
			break;
		case TrianglesInterface.ACTION_BLOCKED:
			messageDisplay = "Blocked";
			break;
		case TrianglesInterface.ACTION_SPACE:
			// Too annoying
			// messageDisplay = "A space cannot slide";
			break;
		case TrianglesInterface.ACTION_MOVED:
			movesDisplay++;
			break;
		case TrianglesInterface.ACTION_SOLVED:
			if (puzzle.getCheat())
				messageDisplay = "No cheating!!"; //username
			else if (!randomized && handleSolved(movesDisplay,
					cornersValue, tileXValue, tileYValue))
				messageDisplay = "Best solution, congratulations!!"; //username
			else
				messageDisplay = "Solved!";
			//showMessage(messageDisplay);
			break;
		case TrianglesInterface.ACTION_COMPUTED:
			break;
		case TrianglesInterface.ACTION_UNDO:
			movesDisplay--;
			break;
		case TrianglesInterface.ACTION_REDO:
			movesDisplay++;
			break;
		case TrianglesInterface.ACTION_RANDOMIZE:
			movesDisplay = 0;
			randomized = true;
			break;
		case TrianglesInterface.ACTION_SPEED:
			{
				int oldDelay, delay;

				delay = puzzle.getDelay();
				oldDelay = delay;
				if (delay > MAX_SPEED - MIN_SPEED)
					delay = MAX_SPEED - MIN_SPEED;
				if (delay < 0)
					delay = 0;
				if (delay != oldDelay) {
					puzzle.setDelay(delay);
				}
				if (speedChanger != null)
					speedChanger.setValue(MAX_SPEED +
						MIN_SPEED - 1 - delay);
			}
			return;
		case TrianglesInterface.ACTION_DECX:
			movesDisplay = 0;
			tileXValue--;
			printRecord(cornersValue, tileXValue, tileYValue);
			puzzle.setPuzzleSizeX(tileXValue);
			if (tileXChanger != null)
				tileXChanger.setValue(tileXValue);
			break;
		case TrianglesInterface.ACTION_INCX:
			movesDisplay = 0;
			tileXValue++;
			printRecord(cornersValue, tileXValue, tileYValue);
			puzzle.setPuzzleSizeX(tileXValue);
			if (tileXChanger != null)
				tileXChanger.setValue(tileXValue);
			break;
		case TrianglesInterface.ACTION_DECY:
			movesDisplay = 0;
			tileYValue--;
			printRecord(cornersValue, tileXValue, tileYValue);
			puzzle.setPuzzleSizeY(tileYValue);
			if (tileYChanger != null)
				tileYChanger.setValue(tileYValue);
			break;
		case TrianglesInterface.ACTION_INCY:
			movesDisplay = 0;
			tileYValue++;
			printRecord(cornersValue, tileXValue, tileYValue);
			puzzle.setPuzzleSizeY(tileYValue);
			if (tileYChanger != null)
				tileYChanger.setValue(tileYValue);
			break;
		case TrianglesInterface.ACTION_CORNERS:
			movesDisplay = 0;
			cornersValue = !cornersValue;
			printRecord(cornersValue, tileXValue, tileYValue);
			puzzle.setPuzzleMode(cornersValue);
			checkBoxCorners.setSelected(cornersValue);
			break;
		case TrianglesInterface.ACTION_IGNORE:
			messageDisplay = "Clear to start";
			break;
		default:
		}
		messageLabel.setText(messageDisplay);
		movesValueLabel.setText(Integer.toString(movesDisplay));
	}

	public TrianglesFrame(String title) {
		/*JPanel info1Panel = new JPanel();
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();*/
		JPanel startPanel = new JPanel();
		JPanel info2Panel = new JPanel();
		JPanel movesPanel = new JPanel();
		JPanel recordPanel = new JPanel();
		JPanel sliderPanel = new JPanel();
		JPanel otherPanel = new JPanel();
		JPanel canvasPanel = new JPanel();

		try {
			mediumIcon = getIcon(TrianglesInterface.ICONS[TrianglesInterface.Icons.X32.ordinal()], 32, 32);
			largeIcon = getIcon(TrianglesInterface.ICONS[TrianglesInterface.Icons.X48.ordinal()], 48, 48);
		} catch (Exception e) {
			largeIcon = null;
		}
		ioDialog = new IODialog(ComponentUtil.findJFrame(this),
			TrianglesInterface.TITLE + ": IO", "Puzzle Position",
			"DONE");
		readScoreRecords();
		getOpts();
		speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.white);
		if (controlValue) {
			menuBar = new JMenuBar();
			JMenu menu;
			JMenuItem menuItem;

			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			menu = new JMenu("File");
			menuItem = new JMenuItem("Get Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_G);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_W);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Show Buffer");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Get File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Exit");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_X);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Play");
			menuItem = new JMenuItem("Undo");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_U);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Redo");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_R);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Clear");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_C);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Randomize");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_Z);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("(Auto-solve)");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_S);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Corners");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_O);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Speed >");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Slow <");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			soundMenuItem = new JCheckBoxMenuItem("Sound @", sound);
			menu.add(soundMenuItem);
			soundMenuItem.addActionListener(this);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Help");
			menuItem = new JMenuItem("Description ?");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Features !");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("References ^");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("About");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_A);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			//setJMenuBar(menuBar); // needs to be in a panel
			mainPanel.add(menuBar, BorderLayout.PAGE_START);
			/*leftPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			leftPanel.add(new JLabel("Move tile"));
			rightPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			rightPanel.add(new JLabel("Clear"));
			leftPanel.add(getIcon("mouse-l.png", 16, 16));
			rightPanel.add(getIcon("mouse-r.png", 16, 16));
			info1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));
			info1Panel.add(leftPanel);
			info1Panel.add(rightPanel);*/
			movesPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			movesPanel.add(new JLabel("Moves"));
			movesValueLabel.setColumns(5);
			movesValueLabel.setEditable(false);
			movesPanel.add(movesValueLabel);
			recordPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			recordPanel.add(new JLabel("Record"));
			recordValueLabel.setColumns(5);
			recordValueLabel.setEditable(false);
			recordPanel.add(recordValueLabel);
			info2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 5));
			info2Panel.add(movesPanel);
			info2Panel.add(recordPanel);
			sliderPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			tileXChanger = new Spinner("Size X",
				tileXValue, TrianglesInterface.MIN_TILES);
			tileYChanger = new Spinner("Size Y",
				tileYValue, TrianglesInterface.MIN_TILES);
			/*tileXChanger = new Slider("Size X",
				tileXValue, TrianglesInterface.MIN_TILES,
				(tileXValue > MAX_TILES) ? tileXValue :
				MAX_TILES));
			tileYChanger = new Slider("Size Y",
				tileYValue, TrianglesInterface.MIN_TILES,
				((tileYValue > MAX_TILES) ? tileYValue :
				MAX_TILES));*/
			sliderPanel.add(tileXChanger);
			sliderPanel.add(tileYChanger);
			otherPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			otherPanel.add(checkBoxCorners);
			speedChanger = new Spinner("Animation Speed",
				speedValue, MIN_SPEED,
				((speedValue > MAX_SPEED) ? speedValue : MAX_SPEED));
			/*speedChanger = new Slider("Animation Speed",
				speedValue, MIN_SPEED,
				((speedValue > MAX_SPEED) ? speedValue : MAX_SPEED));
			speedChanger.setWidth(150);*/
			otherPanel.add(speedChanger);
			checkBoxCorners.setSelected(cornersValue);
			topPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.CENTER));
			/*topPanel.add(info1Panel);*/
			topPanel.add(info2Panel);
			topPanel.add(sliderPanel);
			topPanel.add(otherPanel);
			topPanel.add(messageLabel);
			startPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			startPanel.add(topPanel);
			secondaryPanel.setLayout(new BorderLayout());
			secondaryPanel.add(startPanel, BorderLayout.PAGE_START);
		}
		puzzle = new Triangles(this,
			cornersValue, tileXValue, tileYValue,
			baseValue,
			tileColor, bgColor,
			MAX_SPEED + MIN_SPEED - 1 - speedValue);
		canvasPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		canvasPanel.add(puzzle);
		secondaryPanel.add(canvasPanel, BorderLayout.CENTER);
		mainPanel.add(secondaryPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		if (controlValue) {
			printRecord(cornersValue, tileXValue, tileYValue);
			tileXChanger.addChangeListener(new
				TileXChangeListener());
			tileYChanger.addChangeListener(new
				TileYChangeListener());
			speedChanger.addChangeListener(new
				SpeedChangeListener());
			checkBoxCorners.addItemListener(new
				CornersItemListener());
		}
		canvasPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent event) {
				puzzle.requestFocus();
			}
		});
		setTitle(title);
		/* puzzle.setSize(getSize().width,
			getSize().height - topPanel.getSize().height); */
		puzzle.setSize(widthValue, heightValue);
	}

	public static void main(String[] args) {
		String laf = UIManager.getCrossPlatformLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(laf);
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
			e.printStackTrace();
			System.exit(1);
		}
		List<Image> iconList = new ArrayList<>();
		for (String icon : TrianglesInterface.ICONS) {
			URL url = TrianglesFrame.class.getResource("/" + icon);
			if (url != null)
				iconList.add(Toolkit.getDefaultToolkit().getImage(url));
		}

		argumentParser = new ArgumentParser(args);
		if (argumentParser.hasOption("?")) {
			System.out.println("usage:" + NL +
"\t[-windowWidth={int}] [-windowHeight={int}]" + NL +
"\t[-windowX={int}] [-windowY={int}]" + NL +
"\t[-tileColor={int}] [-bgColor={int}]" + NL +
"\t[-sound={bool}] [-control={bool}]" + NL +
"\t[-size{x|y}={int}] [-corners={bool}] [-base={int}]" + NL +
"\t[-delay={int}]");
			return;
		}
		TrianglesFrame frame = new TrianglesFrame(TrianglesInterface.TITLE);
		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		if (!iconList.isEmpty()) {
			frame.setIconImages(iconList);
		}
		frame.pack();
		frame.puzzle.requestFocusInWindow();
		frame.setLocation(TrianglesFrame.xValue, TrianglesFrame.yValue);
		frame.setVisible(true);
	}
}
